import os
import shutil
import typer
import subprocess
from multiprocessing import Pool

from .download import has_new_crawl, get_last_modified_date

app = typer.Typer()


@app.command()
def download():
    """
    Use esse comando para baixar os dados do site da Receita Federal Brasileira.
    """
    typer.echo("Verificando se existe algo novo na Receita")
    if has_new_crawl():
        typer.echo("Novo crawleamento disponível, iniciando...")
    else:
        typer.echo("Nenhum novo dado disponível")
        raise typer.Abort()


@app.command()
def upload_zip():
    """
    Use esse comando para enviar o diretório zippado (como foi baixado) para o Google Cloud Storage
    """
    timestamp = get_last_modified_date()
    typer.echo(f"Enviando diretório para GCS com data: {timestamp}")
    subprocess.Popen(
        f"gsutil -m cp zip/*.zip gs://driva-lake/crawlers/RFB/{timestamp}/zip",
        shell=True,
    ).wait()


@app.command()
def extract():
    """
    Use esse comando para extrair os arquivos baixados.
    """
    typer.echo("Extraindo arquivos")
    os.makedirs("extracted", exist_ok=True)
    files = [f"zip/{f}" for f in os.listdir("zip") if f.endswith(".zip")]
    with Pool(processes=4) as pool:
        pool.map(extract_file, files)


def extract_file(file):
    subprocess.Popen(f"unzip {file} -d extracted", shell=True).wait()


@app.command()
def combine():
    """
    Use esse comando para processar o diretório zippado gerando o diretório extraído
    """
    typer.echo(f"Ccombinando dados")
    subprocess.Popen(
        f"chmod +x scripts/handle_zip.sh && ./scripts/handle_zip.sh", shell=True
    ).wait()


@app.command()
def upload_extracted():
    """
    Use esse comando para enviar o diretório extraído (após o processamento básico) para o Google Cloud Storage
    """
    timestamp = get_last_modified_date()
    typer.echo(f"Enviando diretório para GCS com data: {timestamp}")
    subprocess.Popen(
        f"gsutil -m cp extracted/*.csv gs://driva-lake/crawlers/RFB/{timestamp}/extracted",
        shell=True,
    ).wait()


@app.command()
def clean():
    """
    Use esse comando para apagar qualquer dado gerado pela CLI.
    """
    typer.echo("Limpando diretórios")
    shutil.rmtree("extracted")
    shutil.rmtree("zip")


@app.command()
def all():
    """
    Use esse comando para baixar, extrair e combinar todos os dados.
    """
    download()
    upload_zip()
    extract()
    combine()
    upload_extracted()
    clean()


if __name__ == "__main__":
    app()



from multiprocessing import Pool
from typing import List, Set
from datetime import datetime

from pywget import wget
from bs4 import BeautifulSoup
import httpx

URL = "http://200.152.38.155/CNPJ/"
DOWNLOAD_FOLDER = "zip"


def get_last_modified_date() -> str:
    html = httpx.get(URL).text
    soup = BeautifulSoup(html, "lxml")

    last_modified = soup.find_all("tr")[3].find_all("td")[2].text.strip()
    last_modified = datetime.strptime(last_modified, "%Y-%m-%d %H:%M").date()

    last_modified = last_modified.strftime("%Y-%m-%d")
    return last_modified


def get_download_links() -> List[str]:
    html = httpx.get(URL).text
    soup = BeautifulSoup(html, "lxml")
    hrefs = [a.get("href") for a in soup.find_all("a")]
    urls = [URL + href for href in hrefs if href.endswith(".zip")]
    return urls


def get_dates_seen() -> Set[str]:
    res = httpx.get(
        "https://blob-api-uvk2pv2hja-uc.a.run.app/files",
        params={"sender_type": "crawler", "sender_id": "RFB"},
    )
    if res.status_code != 200:
        raise ValueError("Failed to get dates seen")
    already_seen = res.json()
    filtered = set()
    for date in already_seen:
        if date:
            filtered.add(date.split("/")[0])
    return filtered


def has_new_crawl() -> bool:
    return not get_last_modified_date() in get_dates_seen()


def _download(url: str):
    wget.download(url, DOWNLOAD_FOLDER + "/")


def download_all():
    links = get_download_links()

    with Pool() as p:
        p.map(_download, links)
